@@ -12,11 +12,13 @@ import typing as t
 
 from colorama import Fore, Style
 
-from vcspull._internal.config_reader import ConfigReader
+from vcspull._internal.config_reader import DuplicateAwareConfigReader
+from vcspull._internal.private_path import PrivatePath
 from vcspull.config import (
     canonicalize_workspace_path,
     expand_dir,
     find_home_config_files,
+    merge_duplicate_workspace_roots,
     normalize_workspace_roots,
     save_config_yaml,
     workspace_root_label,
@@ -24,6 +26,65 @@ from vcspull.config import (
 
 log = logging.getLogger(__name__)
 
+ConfigScope = t.Literal["system", "user", "project", "external"]
+
+
+def _classify_config_scope(
+    config_path: pathlib.Path,
+    *,
+    cwd: pathlib.Path,
+    home: pathlib.Path,
+) -> ConfigScope:
+    """Determine whether a config lives in user, system, project, or external scope."""
+    resolved = config_path.expanduser().resolve()
+    home = home.expanduser().resolve()
+    cwd = cwd.expanduser().resolve()
+
+    default_user_configs = {
+        (home / ".vcspull.yaml").resolve(),
+        (home / ".vcspull.json").resolve(),
+    }
+    if resolved in default_user_configs:
+        return "user"
+
+    xdg_config_home = (
+        pathlib.Path(os.environ.get("XDG_CONFIG_HOME", home / ".config"))
+        .expanduser()
+        .resolve()
+    )
+    user_config_root = (xdg_config_home / "vcspull").resolve()
+    try:
+        resolved.relative_to(user_config_root)
+    except ValueError:
+        pass
+    else:
+        return "user"
+
+    xdg_config_dirs_value = os.environ.get("XDG_CONFIG_DIRS")
+    if xdg_config_dirs_value:
+        config_dir_bases = [
+            pathlib.Path(entry).expanduser().resolve()
+            for entry in xdg_config_dirs_value.split(os.pathsep)
+            if entry
+        ]
+    else:
+        config_dir_bases = [pathlib.Path("/etc/xdg").resolve()]
+
+    for base in config_dir_bases:
+        candidate = (base / "vcspull").resolve()
+        try:
+            resolved.relative_to(candidate)
+        except ValueError:
+            continue
+        else:
+            return "system"
+
+    try:
+        resolved.relative_to(cwd)
+    except ValueError:
+        return "external"
+    return "project"
+
 
 def get_git_origin_url(repo_path: pathlib.Path) -> str | None:
     """Get the origin URL from a git repository.
@@ -102,6 +163,13 @@ def create_discover_subparser(parser: argparse.ArgumentParser) -> None:
         action="store_true",
         help="Preview changes without writing to config file",
     )
+    parser.add_argument(
+        "--no-merge",
+        dest="merge_duplicates",
+        action="store_false",
+        help="Skip merging duplicate workspace roots before writing",
+    )
+    parser.set_defaults(merge_duplicates=True)
 
 
 def _resolve_workspace_path(
@@ -140,6 +208,8 @@ def discover_repos(
     workspace_root_override: str | None,
     yes: bool,
     dry_run: bool,
+    *,
+    merge_duplicates: bool = True,
 ) -> None:
     """Scan filesystem for git repositories and add to vcspull config.
 
@@ -173,7 +243,7 @@ def discover_repos(
                 Fore.CYAN,
                 Style.RESET_ALL,
                 Fore.BLUE,
-                config_file_path,
+                PrivatePath(config_file_path),
                 Style.RESET_ALL,
             )
         elif len(home_configs) > 1:
@@ -184,45 +254,129 @@ def discover_repos(
         else:
             config_file_path = home_configs[0]
 
-    raw_config: dict[str, t.Any] = {}
+    display_config_path = str(PrivatePath(config_file_path))
+
+    cwd = pathlib.Path.cwd()
+    home = pathlib.Path.home()
+    config_scope = _classify_config_scope(config_file_path, cwd=cwd, home=home)
+    allow_relative_workspace = config_scope == "project"
+
+    raw_config: dict[str, t.Any]
+    duplicate_root_occurrences: dict[str, list[t.Any]]
     if config_file_path.exists() and config_file_path.is_file():
         try:
-            loaded_config = ConfigReader._from_file(config_file_path)
+            (
+                raw_config,
+                duplicate_root_occurrences,
+                _top_level_items,
+            ) = DuplicateAwareConfigReader.load_with_duplicates(config_file_path)
+        except TypeError:
+            log.exception(
+                "Config file %s is not a valid YAML dictionary.",
+                display_config_path,
+            )
+            return
         except Exception:
-            log.exception("Error loading YAML from %s. Aborting.", config_file_path)
+            log.exception(
+                "Error loading YAML from %s. Aborting.",
+                PrivatePath(config_file_path),
+            )
             if log.isEnabledFor(logging.DEBUG):
                 traceback.print_exc()
             return
-
-        if loaded_config is None:
+        if raw_config is None:
             raw_config = {}
-        elif isinstance(loaded_config, dict):
-            raw_config = loaded_config
-        else:
+        elif not isinstance(raw_config, dict):
             log.error(
                 "Config file %s is not a valid YAML dictionary.",
-                config_file_path,
+                display_config_path,
             )
             return
     else:
+        raw_config = {}
+        duplicate_root_occurrences = {}
         log.info(
             "%si%s Config file %s%s%s not found. A new one will be created.",
             Fore.CYAN,
             Style.RESET_ALL,
             Fore.BLUE,
-            config_file_path,
+            display_config_path,
             Style.RESET_ALL,
         )
 
-    cwd = pathlib.Path.cwd()
-    home = pathlib.Path.home()
+    duplicate_merge_conflicts: list[str] = []
+    duplicate_merge_changes = 0
+    duplicate_merge_details: list[tuple[str, int]] = []
 
-    normalization_result = normalize_workspace_roots(
-        raw_config,
-        cwd=cwd,
-        home=home,
-    )
-    raw_config, workspace_map, merge_conflicts, _merge_changes = normalization_result
+    if merge_duplicates:
+        (
+            raw_config,
+            duplicate_merge_conflicts,
+            duplicate_merge_changes,
+            duplicate_merge_details,
+        ) = merge_duplicate_workspace_roots(raw_config, duplicate_root_occurrences)
+        for message in duplicate_merge_conflicts:
+            log.warning(message)
+        if duplicate_merge_changes and duplicate_merge_details:
+            for label, occurrence_count in duplicate_merge_details:
+                log.info(
+                    "%s•%s Merged %s%d%s duplicate entr%s for workspace root %s%s%s",
+                    Fore.BLUE,
+                    Style.RESET_ALL,
+                    Fore.YELLOW,
+                    occurrence_count,
+                    Style.RESET_ALL,
+                    "y" if occurrence_count == 1 else "ies",
+                    Fore.MAGENTA,
+                    label,
+                    Style.RESET_ALL,
+                )
+    elif duplicate_root_occurrences:
+        duplicate_merge_details = [
+            (label, len(values)) for label, values in duplicate_root_occurrences.items()
+        ]
+        for label, occurrence_count in duplicate_merge_details:
+            log.warning(
+                "%s•%s Duplicate workspace root %s%s%s appears %s%d%s time%s; "
+                "skipping merge because --no-merge was provided.",
+                Fore.BLUE,
+                Style.RESET_ALL,
+                Fore.MAGENTA,
+                label,
+                Style.RESET_ALL,
+                Fore.YELLOW,
+                occurrence_count,
+                Style.RESET_ALL,
+                "" if occurrence_count == 1 else "s",
+            )
+
+    explicit_relative_override = workspace_root_override in {".", "./"}
+    preserve_cwd_label = explicit_relative_override or allow_relative_workspace
+
+    if merge_duplicates:
+        (
+            raw_config,
+            workspace_map,
+            merge_conflicts,
+            merge_changes,
+        ) = normalize_workspace_roots(
+            raw_config,
+            cwd=cwd,
+            home=home,
+            preserve_cwd_label=preserve_cwd_label,
+        )
+    else:
+        (
+            _,
+            workspace_map,
+            merge_conflicts,
+            merge_changes,
+        ) = normalize_workspace_roots(
+            raw_config,
+            cwd=cwd,
+            home=home,
+            preserve_cwd_label=preserve_cwd_label,
+        )
 
     for message in merge_conflicts:
         log.warning(message)
@@ -248,7 +402,7 @@ def discover_repos(
                     log.warning(
                         "Could not determine remote URL for git repository "
                         "at %s. Skipping.",
-                        repo_path,
+                        PrivatePath(repo_path),
                     )
                     continue
 
@@ -264,7 +418,7 @@ def discover_repos(
                     log.warning(
                         "Could not determine remote URL for git repository "
                         "at %s. Skipping.",
-                        item,
+                        PrivatePath(item),
                     )
                     continue
 
@@ -277,7 +431,7 @@ def discover_repos(
             Fore.YELLOW,
             Style.RESET_ALL,
             Fore.BLUE,
-            scan_dir,
+            PrivatePath(scan_dir),
             Style.RESET_ALL,
         )
         return
@@ -288,7 +442,12 @@ def discover_repos(
     for name, url, workspace_path in found_repos:
         workspace_label = workspace_map.get(workspace_path)
         if workspace_label is None:
-            workspace_label = workspace_root_label(workspace_path, cwd=cwd, home=home)
+            workspace_label = workspace_root_label(
+                workspace_path,
+                cwd=cwd,
+                home=home,
+                preserve_cwd_label=preserve_cwd_label,
+            )
             workspace_map[workspace_path] = workspace_label
             raw_config.setdefault(workspace_label, {})
 
@@ -326,6 +485,7 @@ def discover_repos(
                         workspace_path,
                         cwd=cwd,
                         home=home,
+                        preserve_cwd_label=preserve_cwd_label,
                     )
                     workspace_map[workspace_path] = workspace_label
                     raw_config.setdefault(workspace_label, {})
@@ -344,11 +504,13 @@ def discover_repos(
                     name,
                     Style.RESET_ALL,
                     Fore.BLUE,
-                    config_file_path,
+                    display_config_path,
                     Style.RESET_ALL,
                 )
 
-    changes_made = _merge_changes > 0
+    changes_made = merge_duplicates and (
+        merge_changes > 0 or duplicate_merge_changes > 0
+    )
 
     if not repos_to_add:
         if existing_repos:
@@ -368,11 +530,14 @@ def discover_repos(
                     Fore.GREEN,
                     Style.RESET_ALL,
                     Fore.BLUE,
-                    config_file_path,
+                    display_config_path,
                     Style.RESET_ALL,
                 )
             except Exception:
-                log.exception("Error saving config to %s", config_file_path)
+                log.exception(
+                    "Error saving config to %s",
+                    PrivatePath(config_file_path),
+                )
                 if log.isEnabledFor(logging.DEBUG):
                     traceback.print_exc()
             return
@@ -406,7 +571,7 @@ def discover_repos(
             Fore.YELLOW,
             Style.RESET_ALL,
             Fore.BLUE,
-            config_file_path,
+            display_config_path,
             Style.RESET_ALL,
         )
         return
@@ -422,7 +587,12 @@ def discover_repos(
     for repo_name, repo_url, workspace_path in repos_to_add:
         workspace_label = workspace_map.get(workspace_path)
         if workspace_label is None:
-            workspace_label = workspace_root_label(workspace_path, cwd=cwd, home=home)
+            workspace_label = workspace_root_label(
+                workspace_path,
+                cwd=cwd,
+                home=home,
+                preserve_cwd_label=preserve_cwd_label,
+            )
             workspace_map[workspace_path] = workspace_label
 
         if workspace_label not in raw_config:
@@ -461,11 +631,14 @@ def discover_repos(
                 Fore.GREEN,
                 Style.RESET_ALL,
                 Fore.BLUE,
-                config_file_path,
+                display_config_path,
                 Style.RESET_ALL,
             )
         except Exception:
-            log.exception("Error saving config to %s", config_file_path)
+            log.exception(
+                "Error saving config to %s",
+                PrivatePath(config_file_path),
+            )
             if log.isEnabledFor(logging.DEBUG):
                 traceback.print_exc()
             return
