@@ -2,6 +2,7 @@
 
 from __future__ import annotations
 
+import argparse
 import asyncio
 import contextlib
 import json
@@ -20,11 +21,14 @@ from io import StringIO
 from time import perf_counter
 
 from libvcs._internal.shortcuts import create_project
+from libvcs._internal.types import VCSLiteral
+from libvcs.sync.git import GitSync
 from libvcs.url import registry as url_tools
 
 from vcspull import exc
+from vcspull._internal.private_path import PrivatePath
 from vcspull.config import filter_repos, find_config_files, load_configs
-from vcspull.util import contract_user_home
+from vcspull.types import ConfigDict
 
 from ._colors import Colors, get_color_mode
 from ._output import (
@@ -40,15 +44,6 @@ from ._output import (
 from ._workspaces import filter_by_workspace
 from .status import check_repo_status
 
-if t.TYPE_CHECKING:
-    import argparse
-    import pathlib
-
-    from libvcs._internal.types import VCSLiteral
-    from libvcs.sync.git import GitSync
-
-    from vcspull.types import ConfigDict
-
 log = logging.getLogger(__name__)
 
 ProgressCallback = Callable[[str, datetime], None]
@@ -114,7 +109,7 @@ class PlanProgressPrinter:
                 self._colors.muted(f"✓:{summary.unchanged}"),
                 self._colors.warning(f"⚠:{summary.blocked}"),
                 self._colors.error(f"✗:{summary.errors}"),
-            )
+            ),
         )
         clean_len = _visible_length(line)
         padding = max(self._last_render_len - clean_len, 0)
@@ -145,7 +140,7 @@ def _get_repo_path(repo: ConfigDict) -> pathlib.Path:
     """Return the resolved filesystem path for a repository entry."""
     raw_path = repo.get("path")
     if raw_path is None:
-        return pathlib.Path().resolve()
+        return pathlib.Path.cwd()
     return pathlib.Path(str(raw_path)).expanduser()
 
 
@@ -265,7 +260,7 @@ def _build_plan_entry(
 
     return PlanEntry(
         name=str(repo.get("name", "unknown")),
-        path=contract_user_home(repo_path),
+        path=str(PrivatePath(repo_path)),
         workspace_root=workspace_root,
         action=action,
         detail=detail,
@@ -437,7 +432,7 @@ def _render_plan(
                     display_path = entry.path
             else:
                 # Contract home directory for privacy/brevity in human output
-                display_path = contract_user_home(display_path)
+                display_path = str(PrivatePath(display_path))
 
             detail_text = _format_detail_text(
                 entry,
@@ -459,7 +454,7 @@ def _render_plan(
                     extra_lines.append(f"url: {entry.url}")
                 if entry.ahead is not None or entry.behind is not None:
                     extra_lines.append(
-                        f"ahead/behind: {entry.ahead or 0}/{entry.behind or 0}"
+                        f"ahead/behind: {entry.ahead or 0}/{entry.behind or 0}",
                     )
                 if entry.error:
                     extra_lines.append(f"error: {entry.error}")
@@ -501,9 +496,9 @@ def _emit_plan_output(
             formatter.emit(entry)
         formatter.emit(plan.summary)
         return
-
     structured = PlanResult(entries=display_entries, summary=plan.summary)
-    print(json.dumps(structured.to_json_object(), indent=2))
+    sys.stdout.write(json.dumps(structured.to_json_object(), indent=2) + "\n")
+    sys.stdout.flush()
 
 
 def create_sync_subparser(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
@@ -679,7 +674,7 @@ def sync(
                 found_repos,
                 config=plan_config,
                 progress=progress_printer if progress_enabled else None,
-            )
+            ),
         )
         plan_result.summary.duration_ms = int((perf_counter() - start_time) * 1000)
         if progress_enabled:
@@ -711,7 +706,7 @@ def sync(
 
         def silent_progress(output: str, timestamp: datetime) -> None:
             """Suppress progress for machine-readable output."""
-            return None
+            return
 
         progress_callback = silent_progress
 
@@ -719,13 +714,14 @@ def sync(
         repo_name = repo.get("name", "unknown")
         repo_path = repo.get("path", "unknown")
         workspace_label = repo.get("workspace_root", "")
+        display_repo_path = str(PrivatePath(repo_path))
 
         summary["total"] += 1
 
         event: dict[str, t.Any] = {
             "reason": "sync",
             "name": repo_name,
-            "path": contract_user_home(repo_path),
+            "path": display_repo_path,
             "workspace_root": str(workspace_label),
         }
 
@@ -755,7 +751,8 @@ def sync(
             formatter.emit(event)
             if is_human:
                 log.info(
-                    f"Failed syncing {repo_name}",
+                    "Failed syncing %s",
+                    repo_name,
                 )
             if log.isEnabledFor(logging.DEBUG):
                 import traceback
@@ -770,7 +767,7 @@ def sync(
                     {
                         "reason": "summary",
                         **summary,
-                    }
+                    },
                 )
                 formatter.finalize()
                 if parser is not None:
@@ -783,14 +780,14 @@ def sync(
         formatter.emit(event)
         formatter.emit_text(
             f"{colors.success('✓')} Synced {colors.info(repo_name)} "
-            f"{colors.muted('→')} {repo_path}",
+            f"{colors.muted('→')} {display_repo_path}",
         )
 
     formatter.emit(
         {
             "reason": "summary",
             **summary,
-        }
+        },
     )
 
     if formatter.mode == OutputMode.HUMAN:
