@@ -8,24 +8,19 @@ import logging
 import os
 import pathlib
 import typing as t
+from collections.abc import Callable
 
 from libvcs.sync.git import GitRemote
 
 from vcspull.validator import is_valid_config
 
 from . import exc
-from ._internal.config_reader import ConfigReader
+from ._internal.config_reader import ConfigReader, DuplicateAwareConfigReader
+from .types import ConfigDict, RawConfigDict
 from .util import get_config_dir, update_dict
 
 log = logging.getLogger(__name__)
 
-if t.TYPE_CHECKING:
-    from collections.abc import Callable
-
-    from typing_extensions import TypeGuard
-
-    from .types import ConfigDict, RawConfigDict
-
 
 def expand_dir(
     dir_: pathlib.Path,
@@ -35,14 +30,15 @@ def expand_dir(
 
     Parameters
     ----------
-    _dir : pathlib.Path
+    dir_ : pathlib.Path
+        Directory path to expand
     cwd : pathlib.Path, optional
-        current working dir (for deciphering relative _dir paths), defaults to
-        :py:meth:`os.getcwd()`
+        Current working dir (used to resolve relative paths). Defaults to
+        :py:meth:`pathlib.Path.cwd`.
 
     Returns
     -------
-    pathlib.Path :
+    pathlib.Path
         Absolute directory path
     """
     dir_ = pathlib.Path(os.path.expandvars(str(dir_))).expanduser()
@@ -137,7 +133,7 @@ def extract_repos(
                             **url,
                         )
 
-            def is_valid_config_dict(val: t.Any) -> TypeGuard[ConfigDict]:
+            def is_valid_config_dict(val: t.Any) -> t.TypeGuard[ConfigDict]:
                 assert isinstance(val, dict)
                 return True
 
@@ -242,6 +238,8 @@ def find_config_files(
 def load_configs(
     files: list[pathlib.Path],
     cwd: pathlib.Path | Callable[[], pathlib.Path] = pathlib.Path.cwd,
+    *,
+    merge_duplicates: bool = True,
 ) -> list[ConfigDict]:
     """Return repos from a list of files.
 
@@ -269,9 +267,44 @@ def load_configs(
         if isinstance(file, str):
             file = pathlib.Path(file)
         assert isinstance(file, pathlib.Path)
-        conf = ConfigReader._from_file(file)
-        assert is_valid_config(conf)
-        newrepos = extract_repos(conf, cwd=cwd)
+
+        config_content, duplicate_roots, _top_level_items = (
+            DuplicateAwareConfigReader.load_with_duplicates(file)
+        )
+
+        if merge_duplicates:
+            (
+                config_content,
+                merge_conflicts,
+                _merge_change_count,
+                merge_details,
+            ) = merge_duplicate_workspace_roots(config_content, duplicate_roots)
+
+            for conflict in merge_conflicts:
+                log.warning("%s: %s", file, conflict)
+
+            for root_label, occurrence_count in merge_details:
+                duplicate_count = max(occurrence_count - 1, 0)
+                if duplicate_count == 0:
+                    continue
+                plural = "entry" if duplicate_count == 1 else "entries"
+                log.info(
+                    "%s: merged %d duplicate %s for workspace root '%s'",
+                    file,
+                    duplicate_count,
+                    plural,
+                    root_label,
+                )
+        elif duplicate_roots:
+            duplicate_list = ", ".join(sorted(duplicate_roots.keys()))
+            log.warning(
+                "%s: duplicate workspace roots detected (%s); keeping last occurrences",
+                file,
+                duplicate_list,
+            )
+
+        assert is_valid_config(config_content)
+        newrepos = extract_repos(config_content, cwd=cwd)
 
         if not repos:
             repos.extend(newrepos)
@@ -445,6 +478,97 @@ def save_config_yaml(config_file_path: pathlib.Path, data: dict[t.Any, t.Any]) -
     config_file_path.write_text(yaml_content, encoding="utf-8")
 
 
+def save_config_yaml_with_items(
+    config_file_path: pathlib.Path,
+    items: list[tuple[str, t.Any]],
+) -> None:
+    """Persist configuration data while preserving duplicate top-level sections."""
+    documents: list[str] = []
+
+    for label, section in items:
+        dumped = ConfigReader._dump(
+            fmt="yaml",
+            content={label: section},
+            indent=2,
+        ).rstrip()
+        if dumped:
+            documents.append(dumped)
+
+    yaml_content = "\n".join(documents)
+    if yaml_content:
+        yaml_content += "\n"
+
+    config_file_path.write_text(yaml_content, encoding="utf-8")
+
+
+def merge_duplicate_workspace_root_entries(
+    label: str,
+    occurrences: list[t.Any],
+) -> tuple[t.Any, list[str], int]:
+    """Merge duplicate entries for a single workspace root."""
+    conflicts: list[str] = []
+    change_count = max(len(occurrences) - 1, 0)
+
+    if not occurrences:
+        return {}, conflicts, change_count
+
+    if not all(isinstance(entry, dict) for entry in occurrences):
+        conflicts.append(
+            (
+                f"Workspace root '{label}' contains duplicate entries that are not "
+                "mappings. Keeping the last occurrence."
+            ),
+        )
+        return occurrences[-1], conflicts, change_count
+
+    merged: dict[str, t.Any] = {}
+
+    for entry in occurrences:
+        assert isinstance(entry, dict)
+        for repo_name, repo_config in entry.items():
+            if repo_name not in merged:
+                merged[repo_name] = copy.deepcopy(repo_config)
+            elif merged[repo_name] != repo_config:
+                conflicts.append(
+                    (
+                        f"Workspace root '{label}' contains conflicting definitions "
+                        f"for repository '{repo_name}'. Keeping the existing entry."
+                    ),
+                )
+
+    return merged, conflicts, change_count
+
+
+def merge_duplicate_workspace_roots(
+    config_data: dict[str, t.Any],
+    duplicate_roots: dict[str, list[t.Any]],
+) -> tuple[dict[str, t.Any], list[str], int, list[tuple[str, int]]]:
+    """Merge duplicate workspace root sections captured during load."""
+    if not duplicate_roots:
+        return copy.deepcopy(config_data), [], 0, []
+
+    merged_config = copy.deepcopy(config_data)
+    conflicts: list[str] = []
+    change_count = 0
+    details: list[tuple[str, int]] = []
+
+    for label, occurrences in duplicate_roots.items():
+        (
+            merged_value,
+            entry_conflicts,
+            entry_changes,
+        ) = merge_duplicate_workspace_root_entries(
+            label,
+            occurrences,
+        )
+        merged_config[label] = merged_value
+        conflicts.extend(entry_conflicts)
+        change_count += entry_changes
+        details.append((label, len(occurrences)))
+
+    return merged_config, conflicts, change_count, details
+
+
 def canonicalize_workspace_path(
     label: str,
     *,
@@ -461,14 +585,18 @@ def workspace_root_label(
     *,
     cwd: pathlib.Path | None = None,
     home: pathlib.Path | None = None,
+    preserve_cwd_label: bool = True,
 ) -> str:
     """Create a normalized label for a workspace root path."""
     cwd = cwd or pathlib.Path.cwd()
     home = home or pathlib.Path.home()
 
-    if workspace_path == cwd:
+    if preserve_cwd_label and workspace_path == cwd:
         return "./"
 
+    if workspace_path == home:
+        return "~/"
+
     try:
         relative_to_home = workspace_path.relative_to(home)
         label = f"~/{relative_to_home.as_posix()}"
@@ -486,6 +614,7 @@ def normalize_workspace_roots(
     *,
     cwd: pathlib.Path | None = None,
     home: pathlib.Path | None = None,
+    preserve_cwd_label: bool = True,
 ) -> tuple[dict[str, t.Any], dict[pathlib.Path, str], list[str], int]:
     """Normalize workspace root labels and merge duplicate sections."""
     cwd = cwd or pathlib.Path.cwd()
@@ -505,6 +634,7 @@ def normalize_workspace_roots(
                 canonical_path,
                 cwd=cwd,
                 home=home,
+                preserve_cwd_label=preserve_cwd_label,
             )
             path_to_label[canonical_path] = normalized_label
 
@@ -526,18 +656,19 @@ def normalize_workspace_roots(
                         change_count += 1
                     elif existing_value[repo_name] != repo_config:
                         conflict_message = (
-                            "Workspace root '{label}' contains conflicting definitions "
-                            "for repository '{repo}'. Keeping the existing entry."
+                            f"Workspace root '{label}' contains conflicting "
+                            "definitions for repository '{repo}'. Keeping the existing "
+                            "entry."
                         )
                         conflicts.append(
                             conflict_message.format(
                                 label=normalized_label,
                                 repo=repo_name,
-                            )
+                            ),
                         )
             elif existing_value != value:
                 conflict_message = (
-                    "Workspace root '{label}' contains conflicting non-dictionary "
+                    f"Workspace root '{label}' contains conflicting non-dictionary "
                     "values. Keeping the existing entry."
                 )
                 conflicts.append(conflict_message.format(label=normalized_label))
